<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">

<gui name="KDE Partition Manager" version="4">
	<ToolBar name="editToolBar">
		<text context="@title:menu turn on and off edit toolbar">Edit Toolbar</text>
		<Action name="applyAllOperations"/>
		<Action name="undoOperation"/>
		<Action name="clearAllOperations"/>
	</ToolBar>

	<ToolBar name="partitionToolBar">
		<text context="@title:menu">Partition Toolbar</text>
		<Action name="newPartition"/>
		<Action name="resizePartition"/>
		<Action name="copyPartition"/>
		<Action name="pastePartition"/>
		<Action name="deletePartition"/>
		<Action name="checkPartition"/>
		<Action name="propertiesPartition"/>
	</ToolBar>

	<ToolBar name="deviceToolBar">
		<text context="@title:menu">Device Toolbar</text>
		<Action name="createNewPartitionTable"/>
	</ToolBar>

	<MenuBar>
		<Menu name="file"/>
		<Menu name="edit">
			<Action name="undoOperation"/>
			<Action name="clearAllOperations"/>
			<Action name="applyAllOperations"/>
		</Menu>
		<Menu name="view">
			<text context="@title:menu">View</text>
			<Action name="toggleDockDevices"/>
			<Action name="toggleDockOperations"/>
			<Action name="toggleDockInformation"/>
			<Action name="toggleDockLog"/>
			<separator name="separator_7"/>
			<Action name="fileSystemSupport"/>
			<Action name="refreshDevices"/>
		</Menu>
		<Menu name="device">
			<text context="@title:menu">Device</text>
			<Action name="createNewPartitionTable"/>
		</Menu>
		<Menu name="partition">
			<text context="@title:menu">Partition</text>
			<Action name="newPartition"/>
			<Action name="resizePartition"/>
			<Action name="deletePartition"/>
			<separator name="separator_6"/>
			<Action name="copyPartition"/>
			<Action name="pastePartition"/>
			<separator name="separator_3"/>
			<Action name="mountPartition"/>
			<separator name="separator_4"/>
			<Action name="checkPartition"/>
			<separator name="separator_5"/>
			<Action name="backupPartition"/>
			<Action name="restorePartition"/>
			<separator name="separator_8"/>
			<Action name="propertiesPartition"/>
		</Menu>
		<Menu name="settings"/>
		<Menu name="help"/>
	</MenuBar>

	<StatusBar/>

</gui>

